"use strict";
// Copyright 2021-2023 IOTA Stiftung
// SPDX-License-Identifier: Apache-2.0
Object.defineProperty(exports, "__esModule", { value: true });
exports.Client = void 0;
const client_method_handler_1 = require("./client-method-handler");
const client_1 = require("../types/client");
const block_1 = require("../types/block");
const types_1 = require("../types");
const api_1 = require("../types/models/api");
const class_transformer_1 = require("class-transformer");
/** The Client to interact with nodes. */
class Client {
    constructor(options) {
        this.methodHandler = new client_method_handler_1.ClientMethodHandler(options);
    }
    async destroy() {
        return this.methodHandler.destroy();
    }
    /**
     * Returns the node information together with the url of the used node
     * @returns { Promise<INodeInfoWrapper> }.
     */
    async getInfo() {
        const response = await this.methodHandler.callMethod({
            name: 'getInfo',
        });
        return JSON.parse(response).payload;
    }
    /**
     * Gets the network related information such as network_id and min_pow_score
     */
    async getNetworkInfo() {
        const response = await this.methodHandler.callMethod({
            name: 'getNetworkInfo',
        });
        return JSON.parse(response).payload;
    }
    /** Fetch basic output IDs based on query parameters */
    async basicOutputIds(queryParameters) {
        const response = await this.methodHandler.callMethod({
            name: 'basicOutputIds',
            data: {
                queryParameters,
            },
        });
        return JSON.parse(response).payload;
    }
    /** Get output from a known outputID */
    async getOutput(outputId) {
        const response = await this.methodHandler.callMethod({
            name: 'getOutput',
            data: {
                outputId,
            },
        });
        const parsed = JSON.parse(response);
        return (0, class_transformer_1.plainToInstance)(api_1.OutputResponse, parsed.payload);
    }
    /** Fetch OutputResponse from provided OutputIds (requests are sent in parallel) */
    async getOutputs(outputIds) {
        const response = await this.methodHandler.callMethod({
            name: 'getOutputs',
            data: {
                outputIds,
            },
        });
        const parsed = JSON.parse(response);
        return (0, class_transformer_1.plainToInstance)(api_1.OutputResponse, parsed.payload);
    }
    /** Build and post a block */
    async buildAndPostBlock(secretManager, options) {
        if (options &&
            options.output &&
            typeof options.output.amount === 'bigint') {
            options.output.amount = options.output.amount.toString(10);
        }
        if (options &&
            options.outputHex &&
            typeof options.outputHex.amount === 'bigint') {
            options.outputHex.amount = options.outputHex.amount.toString(10);
        }
        const response = await this.methodHandler.callMethod({
            name: 'buildAndPostBlock',
            data: {
                secretManager,
                options,
            },
        });
        const parsed = JSON.parse(response);
        const block = (0, class_transformer_1.plainToInstance)(block_1.Block, parsed.payload[1]);
        return [parsed.payload[0], block];
    }
    /**
     * Returns tips that are ideal for attaching a block.
     * The tips can be considered as non-lazy and are therefore ideal for attaching a block.
     */
    async getTips() {
        const response = await this.methodHandler.callMethod({
            name: 'getTips',
        });
        return JSON.parse(response).payload;
    }
    /**
     * Post block in JSON format, returns the block ID.
     */
    async postBlock(block) {
        const response = await this.methodHandler.callMethod({
            name: 'postBlock',
            data: {
                block,
            },
        });
        return JSON.parse(response).payload;
    }
    /**
     * Get block as JSON.
     */
    async getBlock(blockId) {
        const response = await this.methodHandler.callMethod({
            name: 'getBlock',
            data: {
                blockId,
            },
        });
        const parsed = JSON.parse(response);
        return (0, class_transformer_1.plainToInstance)(block_1.Block, parsed.payload);
    }
    /**
     * Get block metadata.
     */
    async getBlockMetadata(blockId) {
        const response = await this.methodHandler.callMethod({
            name: 'getBlockMetadata',
            data: {
                blockId,
            },
        });
        return JSON.parse(response).payload;
    }
    /**
     * Find inputs from addresses for a provided amount (useful for offline signing)
     */
    async findInputs(addresses, amount) {
        const response = await this.methodHandler.callMethod({
            name: 'findInputs',
            data: {
                addresses,
                amount: Number(amount),
            },
        });
        const parsed = JSON.parse(response);
        return (0, class_transformer_1.plainToInstance)(types_1.UTXOInput, parsed.payload);
    }
    /**
     * Prepare a transaction for signing
     */
    async prepareTransaction(secretManager, options) {
        const response = await this.methodHandler.callMethod({
            name: 'prepareTransaction',
            data: {
                secretManager,
                options,
            },
        });
        const parsed = JSON.parse(response);
        return (0, class_transformer_1.plainToInstance)(client_1.PreparedTransactionData, parsed.payload);
    }
    /**
     * Sign a transaction
     */
    async signTransaction(secretManager, preparedTransactionData) {
        const response = await this.methodHandler.callMethod({
            name: 'signTransaction',
            data: {
                secretManager,
                preparedTransactionData,
            },
        });
        const parsed = JSON.parse(response);
        return (0, class_transformer_1.plainToInstance)(block_1.TransactionPayload, parsed.payload);
    }
    /**
     * Create a signature unlock using the provided `secretManager`.
     */
    async signatureUnlock(secretManager, transactionEssenceHash, chain) {
        const response = await this.methodHandler.callMethod({
            name: 'signatureUnlock',
            data: {
                secretManager,
                transactionEssenceHash,
                chain,
            },
        });
        return block_1.UnlockCondition.parse(JSON.parse(response).payload);
    }
    /**
     * Submit a payload in a block
     */
    async postBlockPayload(payload) {
        const response = await this.methodHandler.callMethod({
            name: 'postBlockPayload',
            data: {
                payload,
            },
        });
        const parsed = JSON.parse(response);
        const block = (0, class_transformer_1.plainToInstance)(block_1.Block, parsed.payload[1]);
        return [parsed.payload[0], block];
    }
    /**
     * Get a node candidate from the healthy node pool.
     */
    async getNode() {
        const response = await this.methodHandler.callMethod({
            name: 'getNode',
        });
        return JSON.parse(response).payload;
    }
    /**
     * Get the network id of the node we're connecting to.
     */
    async getNetworkId() {
        const response = await this.methodHandler.callMethod({
            name: 'getNetworkId',
        });
        return JSON.parse(response).payload;
    }
    /**
     * Returns the bech32_hrp.
     */
    async getBech32Hrp() {
        const response = await this.methodHandler.callMethod({
            name: 'getBech32Hrp',
        });
        return JSON.parse(response).payload;
    }
    /**
     * Returns the min PoW score.
     */
    async getMinPowScore() {
        const response = await this.methodHandler.callMethod({
            name: 'getMinPowScore',
        });
        return JSON.parse(response).payload;
    }
    /**
     * Returns the tips interval.
     */
    async getTipsInterval() {
        const response = await this.methodHandler.callMethod({
            name: 'getTipsInterval',
        });
        return JSON.parse(response).payload;
    }
    /**
     * Returns the token supply.
     */
    async getTokenSupply() {
        return (await this.getProtocolParameters()).tokenSupply;
    }
    /**
     * Returns the protocol parameters.
     */
    async getProtocolParameters() {
        const response = await this.methodHandler.callMethod({
            name: 'getProtocolParameters',
        });
        return JSON.parse(response).payload;
    }
    /**
     * Returns if local pow should be used or not.
     */
    async getLocalPow() {
        const response = await this.methodHandler.callMethod({
            name: 'getLocalPow',
        });
        return JSON.parse(response).payload;
    }
    /**
     * Get fallback to local proof of work timeout.
     */
    async getFallbackToLocalPow() {
        const response = await this.methodHandler.callMethod({
            name: 'getFallbackToLocalPow',
        });
        return JSON.parse(response).payload;
    }
    /**
     * Get health of node by input url.
     */
    async getHealth(url) {
        const response = await this.methodHandler.callMethod({
            name: 'getHealth',
            data: {
                url,
            },
        });
        return JSON.parse(response).payload;
    }
    /**
     * Get info of node with input url.
     */
    async getNodeInfo(url, auth) {
        const response = await this.methodHandler.callMethod({
            name: 'getNodeInfo',
            data: {
                url,
                auth,
            },
        });
        return JSON.parse(response).payload;
    }
    /**
     * Get peers.
     */
    async getPeers() {
        const response = await this.methodHandler.callMethod({
            name: 'getPeers',
        });
        return JSON.parse(response).payload;
    }
    /**
     * Post block as raw bytes, returns the block ID.
     */
    async postBlockRaw(block) {
        const response = await this.methodHandler.callMethod({
            name: 'postBlockRaw',
            data: {
                block,
            },
        });
        return JSON.parse(response).payload;
    }
    /**
     * Get block as raw bytes.
     */
    async getBlockRaw(blockId) {
        const response = await this.methodHandler.callMethod({
            name: 'getBlockRaw',
            data: {
                blockId,
            },
        });
        return JSON.parse(response).payload;
    }
    /**
     * Look up a milestone by a given milestone index.
     */
    async getMilestoneById(milestoneId) {
        const response = await this.methodHandler.callMethod({
            name: 'getMilestoneById',
            data: {
                milestoneId,
            },
        });
        const parsed = JSON.parse(response);
        return (0, class_transformer_1.plainToInstance)(block_1.MilestonePayload, parsed.payload);
    }
    /**
     * Returns all UTXO changes that happened at a specific milestone.
     */
    async getUtxoChangesById(milestoneId) {
        const response = await this.methodHandler.callMethod({
            name: 'getUtxoChangesById',
            data: {
                milestoneId,
            },
        });
        return JSON.parse(response).payload;
    }
    /**
     * Look up a milestone by a given milestone index.
     */
    async getMilestoneByIndex(index) {
        const response = await this.methodHandler.callMethod({
            name: 'getMilestoneByIndex',
            data: {
                index,
            },
        });
        const parsed = JSON.parse(response);
        return (0, class_transformer_1.plainToInstance)(block_1.MilestonePayload, parsed.payload);
    }
    /**
     * Returns all UTXO changes that happened at a specific milestone.
     */
    async getUtxoChangesByIndex(index) {
        const response = await this.methodHandler.callMethod({
            name: 'getUtxoChangesByIndex',
            data: {
                index,
            },
        });
        return JSON.parse(response).payload;
    }
    /**
     * Get receipts.
     */
    async getReceipts() {
        const response = await this.methodHandler.callMethod({
            name: 'getReceipts',
        });
        const parsed = JSON.parse(response);
        return (0, class_transformer_1.plainToInstance)(api_1.ReceiptsResponse, parsed.payload);
    }
    /**
     * Get the receipts by the given milestone index.
     */
    async getReceiptsMigratedAt(milestoneIndex) {
        const response = await this.methodHandler.callMethod({
            name: 'getReceiptsMigratedAt',
            data: {
                milestoneIndex,
            },
        });
        const parsed = JSON.parse(response);
        return (0, class_transformer_1.plainToInstance)(api_1.ReceiptsResponse, parsed.payload);
    }
    /**
     * Get the treasury output.
     */
    async getTreasury() {
        const response = await this.methodHandler.callMethod({
            name: 'getTreasury',
        });
        return block_1.Output.parse(JSON.parse(response).payload);
    }
    /**
     * Returns the included block of the transaction.
     */
    async getIncludedBlock(transactionId) {
        const response = await this.methodHandler.callMethod({
            name: 'getIncludedBlock',
            data: {
                transactionId,
            },
        });
        const parsed = JSON.parse(response);
        return (0, class_transformer_1.plainToInstance)(block_1.Block, parsed.payload);
    }
    /**
     * Returns the metadata of the included block of the transaction.
     */
    async getIncludedBlockMetadata(transactionId) {
        const response = await this.methodHandler.callMethod({
            name: 'getIncludedBlockMetadata',
            data: {
                transactionId,
            },
        });
        const parsed = JSON.parse(response);
        return (0, class_transformer_1.plainToInstance)(block_1.Block, parsed.payload);
    }
    /**
     * Transforms a hex encoded address to a bech32 encoded address.
     */
    async hexToBech32(hex, bech32Hrp) {
        const response = await this.methodHandler.callMethod({
            name: 'hexToBech32',
            data: {
                hex,
                bech32Hrp,
            },
        });
        return JSON.parse(response).payload;
    }
    /**
     * Transforms an alias id to a bech32 encoded address.
     */
    async aliasIdToBech32(aliasId, bech32Hrp) {
        const response = await this.methodHandler.callMethod({
            name: 'aliasIdToBech32',
            data: {
                aliasId,
                bech32Hrp,
            },
        });
        return JSON.parse(response).payload;
    }
    /**
     * Transforms an nft id to a bech32 encoded address.
     */
    async nftIdToBech32(nftId, bech32Hrp) {
        const response = await this.methodHandler.callMethod({
            name: 'nftIdToBech32',
            data: {
                nftId,
                bech32Hrp,
            },
        });
        return JSON.parse(response).payload;
    }
    /**
     * Transforms a hex encoded public key to a bech32 encoded address.
     */
    async hexPublicKeyToBech32Address(hex, bech32Hrp) {
        const response = await this.methodHandler.callMethod({
            name: 'hexPublicKeyToBech32Address',
            data: {
                hex,
                bech32Hrp,
            },
        });
        return JSON.parse(response).payload;
    }
    /**
     * Fetch alias output IDs
     */
    async aliasOutputIds(queryParameters) {
        const response = await this.methodHandler.callMethod({
            name: 'aliasOutputIds',
            data: {
                queryParameters,
            },
        });
        return JSON.parse(response).payload;
    }
    /**
     * Fetch alias output ID
     */
    async aliasOutputId(aliasId) {
        const response = await this.methodHandler.callMethod({
            name: 'aliasOutputId',
            data: {
                aliasId,
            },
        });
        return JSON.parse(response).payload;
    }
    /**
     * Fetch NFT output IDs
     */
    async nftOutputIds(queryParameters) {
        const response = await this.methodHandler.callMethod({
            name: 'nftOutputIds',
            data: {
                queryParameters,
            },
        });
        return JSON.parse(response).payload;
    }
    /**
     * Fetch NFT output ID
     */
    async nftOutputId(nftId) {
        const response = await this.methodHandler.callMethod({
            name: 'nftOutputId',
            data: {
                nftId,
            },
        });
        return JSON.parse(response).payload;
    }
    /**
     * Fetch Foundry Output IDs
     */
    async foundryOutputIds(queryParameters) {
        const response = await this.methodHandler.callMethod({
            name: 'foundryOutputIds',
            data: {
                queryParameters,
            },
        });
        return JSON.parse(response).payload;
    }
    /**
     * Fetch Foundry Output ID
     */
    async foundryOutputId(foundryId) {
        const response = await this.methodHandler.callMethod({
            name: 'foundryOutputId',
            data: {
                foundryId,
            },
        });
        return JSON.parse(response).payload;
    }
    /**
     * Try to get OutputResponse from provided OutputIds (requests are sent
     * in parallel and errors are ignored, can be useful for spent outputs)
     */
    async getOutputsIgnoreErrors(outputIds) {
        const response = await this.methodHandler.callMethod({
            name: 'getOutputsIgnoreErrors',
            data: {
                outputIds,
            },
        });
        const parsed = JSON.parse(response);
        return (0, class_transformer_1.plainToInstance)(api_1.OutputResponse, parsed.payload);
    }
    /**
     * Find all blocks by provided block IDs.
     */
    async findBlocks(blockIds) {
        const response = await this.methodHandler.callMethod({
            name: 'findBlocks',
            data: {
                blockIds,
            },
        });
        const parsed = JSON.parse(response);
        return (0, class_transformer_1.plainToInstance)(block_1.Block, parsed.payload);
    }
    /**
     * Retries (promotes or reattaches) a block for provided block id. Block should be
     * retried only if they are valid and haven't been confirmed for a while.
     */
    async retry(blockId) {
        const response = await this.methodHandler.callMethod({
            name: 'retry',
            data: {
                blockId,
            },
        });
        const parsed = JSON.parse(response);
        const block = (0, class_transformer_1.plainToInstance)(block_1.Block, parsed.payload[1]);
        return [parsed.payload[0], block];
    }
    /**
     * Retries (promotes or reattaches) a block for provided block id until it's included (referenced by a
     * milestone). Default interval is 5 seconds and max attempts is 40. Returns the included block at first
     * position and additional reattached blocks
     */
    async retryUntilIncluded(blockId, interval, maxAttempts) {
        const response = await this.methodHandler.callMethod({
            name: 'retryUntilIncluded',
            data: {
                blockId,
                interval,
                maxAttempts,
            },
        });
        const parsed = JSON.parse(response);
        const arr = [];
        parsed.payload.forEach((payload) => {
            arr.push([payload[0], (0, class_transformer_1.plainToInstance)(block_1.Block, payload[1])]);
        });
        return arr;
    }
    /**
     * Function to consolidate all funds from a range of addresses to the address with the lowest index in that range
     * Returns the address to which the funds got consolidated, if any were available
     */
    async consolidateFunds(secretManager, generateAddressesOptions) {
        const response = await this.methodHandler.callMethod({
            name: 'consolidateFunds',
            data: {
                secretManager,
                generateAddressesOptions,
            },
        });
        return JSON.parse(response).payload;
    }
    /**
     * Reattaches blocks for provided block id. Blocks can be reattached only if they are valid and haven't been
     * confirmed for a while.
     */
    async reattach(blockId) {
        const response = await this.methodHandler.callMethod({
            name: 'reattach',
            data: {
                blockId,
            },
        });
        const parsed = JSON.parse(response);
        const block = (0, class_transformer_1.plainToInstance)(block_1.Block, parsed.payload[1]);
        return [parsed.payload[0], block];
    }
    /**
     * Reattach a block without checking if it should be reattached
     */
    async reattachUnchecked(blockId) {
        const response = await this.methodHandler.callMethod({
            name: 'reattachUnchecked',
            data: {
                blockId,
            },
        });
        const parsed = JSON.parse(response);
        const block = (0, class_transformer_1.plainToInstance)(block_1.Block, parsed.payload[1]);
        return [parsed.payload[0], block];
    }
    /**
     * Promotes a block. The method should validate if a promotion is necessary through get_block. If not, the
     * method should error out and should not allow unnecessary promotions.
     */
    async promote(blockId) {
        const response = await this.methodHandler.callMethod({
            name: 'promote',
            data: {
                blockId,
            },
        });
        const parsed = JSON.parse(response);
        const block = (0, class_transformer_1.plainToInstance)(block_1.Block, parsed.payload[1]);
        return [parsed.payload[0], block];
    }
    /**
     * Promote a block without checking if it should be promoted
     */
    async promoteUnchecked(blockId) {
        const response = await this.methodHandler.callMethod({
            name: 'promoteUnchecked',
            data: {
                blockId,
            },
        });
        const parsed = JSON.parse(response);
        const block = (0, class_transformer_1.plainToInstance)(block_1.Block, parsed.payload[1]);
        return [parsed.payload[0], block];
    }
    /**
     * Returns the unhealthy nodes.
     */
    async unhealthyNodes() {
        const response = await this.methodHandler.callMethod({
            name: 'unhealthyNodes',
        });
        return JSON.parse(response).payload;
    }
    /**
     * Build a Basic Output.
     */
    async buildBasicOutput(params) {
        if (params.amount && typeof params.amount === 'bigint') {
            params.amount = params.amount.toString(10);
        }
        const response = await this.methodHandler.callMethod({
            name: 'buildBasicOutput',
            data: params,
        });
        const parsed = JSON.parse(response);
        return (0, class_transformer_1.plainToInstance)(block_1.BasicOutput, parsed.payload);
    }
    /**
     * Build an Alias Output.
     */
    async buildAliasOutput(params) {
        if (params.amount && typeof params.amount === 'bigint') {
            params.amount = params.amount.toString(10);
        }
        const response = await this.methodHandler.callMethod({
            name: 'buildAliasOutput',
            data: params,
        });
        const parsed = JSON.parse(response);
        return (0, class_transformer_1.plainToInstance)(block_1.AliasOutput, parsed.payload);
    }
    /**
     * Build a Foundry Output.
     */
    async buildFoundryOutput(params) {
        if (params.amount && typeof params.amount === 'bigint') {
            params.amount = params.amount.toString(10);
        }
        const response = await this.methodHandler.callMethod({
            name: 'buildFoundryOutput',
            data: params,
        });
        const parsed = JSON.parse(response);
        return (0, class_transformer_1.plainToInstance)(block_1.FoundryOutput, parsed.payload);
    }
    /**
     * Build an Nft Output.
     */
    async buildNftOutput(params) {
        if (params.amount && typeof params.amount === 'bigint') {
            params.amount = params.amount.toString(10);
        }
        const response = await this.methodHandler.callMethod({
            name: 'buildNftOutput',
            data: params,
        });
        const parsed = JSON.parse(response);
        return (0, class_transformer_1.plainToInstance)(block_1.NftOutput, parsed.payload);
    }
    /**
     * Listen to MQTT topics.
     */
    async listenMqtt(topics, callback) {
        return this.methodHandler.listen(topics, callback);
    }
    /**
     * Stop listening for provided MQTT topics.
     */
    async clearMqttListeners(topics) {
        await this.methodHandler.callMethod({
            name: 'clearListeners',
            data: {
                topics,
            },
        });
    }
    /**
     * Calculate the minimum required storage deposit for an output.
     * @param output output to calculate the deposit amount for.
     * @returns The amount.
     */
    async minimumRequiredStorageDeposit(output) {
        const response = await this.methodHandler.callMethod({
            name: 'minimumRequiredStorageDeposit',
            data: {
                output,
            },
        });
        return JSON.parse(response).payload;
    }
    /**
     * Request funds from a faucet, for example `https://faucet.testnet.shimmer.network/api/enqueue` or `http://localhost:8091/api/enqueue`.
     */
    async requestFundsFromFaucet(url, address) {
        const response = await this.methodHandler.callMethod({
            name: 'requestFundsFromFaucet',
            data: { url, address },
        });
        return JSON.parse(response).payload;
    }
    /**
     * Extension method which provides request methods for plugins.
     * @param basePluginPath The base path for the plugin eg indexer/v1/ .
     * @param method The http method.
     * @param endpoint The path for the plugin request.
     * @param queryParams Additional query params for the request.
     * @param request The request object.
     * @returns The response json.
     */
    async callPluginRoute(basePluginPath, method, endpoint, queryParams, request) {
        const response = await this.methodHandler.callMethod({
            name: 'callPluginRoute',
            data: {
                basePluginPath,
                method,
                endpoint,
                queryParams: queryParams ?? [],
                request,
            },
        });
        return JSON.parse(response).payload;
    }
}
exports.Client = Client;
//# sourceMappingURL=client.js.map