import { UnlockCondition } from './unlock-condition';
import { Feature } from './feature';
import { HexEncodedString } from '../../utils';
import { TokenScheme } from './token-scheme';
import { INativeToken } from '../../models';
export type OutputId = string;
/**
 * All of the output types.
 */
declare enum OutputType {
    Treasury = 2,
    Basic = 3,
    Alias = 4,
    Foundry = 5,
    Nft = 6
}
declare abstract class Output {
    private amount;
    private type;
    constructor(type: OutputType, amount: bigint | string);
    /**
     * The type of output.
     */
    getType(): OutputType;
    /**
     * The amount of the output.
     */
    getAmount(): bigint;
    static parse(data: any): Output;
}
/**
 * Common output properties.
 */
declare abstract class CommonOutput extends Output {
    private unlockConditions;
    private nativeTokens?;
    private features?;
    constructor(type: OutputType, amount: bigint, unlockConditions: UnlockCondition[]);
    /**
     * The unlock conditions for the output.
     */
    getUnlockConditions(): UnlockCondition[];
    /**
     * The native tokens held by the output.
     */
    getNativeTokens(): INativeToken[] | undefined;
    /**
     * Features contained by the output.
     */
    getFeatures(): Feature[] | undefined;
}
/**
 * Treasury Output.
 */
declare class TreasuryOutput extends Output {
    constructor(amount: bigint);
}
/**
 * Basic output.
 */
declare class BasicOutput extends CommonOutput {
    constructor(amount: bigint, unlockConditions: UnlockCondition[]);
}
declare abstract class ImmutableFeaturesOutput extends CommonOutput {
    private immutableFeatures?;
    constructor(type: OutputType, amount: bigint, unlockConditions: UnlockCondition[]);
    /**
     * Immutable features contained by the output.
     */
    getImmutableFeatures(): Feature[] | undefined;
}
declare abstract class StateMetadataOutput extends ImmutableFeaturesOutput {
    private stateMetadata?;
    constructor(type: OutputType, amount: bigint, unlockConditions: UnlockCondition[]);
    /**
     * Metadata that can only be changed by the state controller.
     */
    getStateMetadata(): HexEncodedString | undefined;
}
declare class AliasOutput extends StateMetadataOutput {
    private aliasId;
    private stateIndex;
    private foundryCounter;
    constructor(unlockConditions: UnlockCondition[], amount: bigint, aliasId: HexEncodedString, stateIndex: number, foundryCounter: number);
    /**
     * Unique identifier of the alias, which is the BLAKE2b-160 hash of the Output ID that created it.
     * Unless its a newly created alias, then the id is zeroed.
     */
    getAliasId(): HexEncodedString;
    /**
     * A counter that must increase by 1 every time the alias is state transitioned.
     */
    getStateIndex(): number;
    /**
     * A counter that denotes the number of foundries created by this alias account.
     */
    getFoundryCounter(): number;
}
/**
 * NFT output.
 */
declare class NftOutput extends ImmutableFeaturesOutput {
    private nftId;
    constructor(amount: bigint, nftId: HexEncodedString, unlockConditions: UnlockCondition[]);
    /**
     * Unique identifier of the NFT, which is the BLAKE2b-160 hash of the Output ID that created it.
     * Unless its newly minted, then the id is zeroed.
     */
    getNftId(): HexEncodedString;
}
/**
 * Foundry output.
 */
declare class FoundryOutput extends ImmutableFeaturesOutput {
    private serialNumber;
    private tokenScheme;
    constructor(amount: bigint, serialNumber: number, unlockConditions: UnlockCondition[], tokenScheme: TokenScheme);
    /**
     * The serial number of the foundry with respect to the controlling alias.
     */
    getSerialNumber(): number;
    /**
     * The token scheme for the foundry.
     */
    getTokenScheme(): TokenScheme;
}
declare const OutputDiscriminator: {
    property: string;
    subTypes: ({
        value: typeof TreasuryOutput;
        name: any;
    } | {
        value: typeof BasicOutput;
        name: any;
    } | {
        value: typeof AliasOutput;
        name: any;
    } | {
        value: typeof NftOutput;
        name: any;
    } | {
        value: typeof FoundryOutput;
        name: any;
    })[];
};
export { OutputDiscriminator, Output, OutputType, CommonOutput, TreasuryOutput, BasicOutput, AliasOutput, NftOutput, FoundryOutput, };
