"use strict";
// Copyright 2023 IOTA Stiftung
// SPDX-License-Identifier: Apache-2.0
Object.defineProperty(exports, "__esModule", { value: true });
exports.Utils = void 0;
const bindings_1 = require("../bindings");
const types_1 = require("../types");
/** Utils class for utils. */
class Utils {
    /**
     * Generates a new mnemonic.
     */
    static generateMnemonic() {
        return (0, bindings_1.callUtilsMethod)({
            name: 'generateMnemonic',
        });
    }
    /**
     * Returns a hex encoded seed for a mnemonic.
     */
    static mnemonicToHexSeed(mnemonic) {
        return (0, bindings_1.callUtilsMethod)({
            name: 'mnemonicToHexSeed',
            data: {
                mnemonic,
            },
        });
    }
    /**
     * Computes the alias id for the given alias output id.
     */
    static computeAliasId(outputId) {
        return (0, bindings_1.callUtilsMethod)({
            name: 'computeAliasId',
            data: {
                outputId,
            },
        });
    }
    /**
     * Computes the foundry id.
     */
    static computeFoundryId(aliasId, serialNumber, tokenSchemeKind) {
        return (0, bindings_1.callUtilsMethod)({
            name: 'computeFoundryId',
            data: {
                aliasId,
                serialNumber,
                tokenSchemeKind,
            },
        });
    }
    /**
     * Computes the NFT id for the given NFT output id.
     */
    static computeNftId(outputId) {
        return (0, bindings_1.callUtilsMethod)({
            name: 'computeNftId',
            data: {
                outputId,
            },
        });
    }
    /**
     * Computes the inputCommitment from the output objects that are used as inputs to fund the transaction.
     * @param inputs The output objects used as inputs for the transaction.
     * @returns The inputs commitment.
     */
    static computeInputsCommitment(inputs) {
        return (0, bindings_1.callUtilsMethod)({
            name: 'computeInputsCommitment',
            data: {
                inputs,
            },
        });
    }
    /**
     * Computes the output ID from transaction id and output index.
     * @param transactionId The id of the transaction.
     * @param outputIndex The index of the output.
     * @returns The output id.
     */
    static computeOutputId(id, index) {
        return (0, bindings_1.callUtilsMethod)({
            name: 'computeOutputId',
            data: {
                id,
                index,
            },
        });
    }
    /**
     * Computes the required storage deposit of an output.
     * @param output The output.
     * @param rentStructure Rent cost of objects which take node resources.
     * @returns The required storage deposit.
     */
    static computeStorageDeposit(output, rentStructure) {
        const depositHex = (0, bindings_1.callUtilsMethod)({
            name: 'computeStorageDeposit',
            data: {
                output,
                rentStructure,
            },
        });
        return (0, types_1.hexToBigInt)(depositHex);
    }
    /**
     * Computes a tokenId from the aliasId, serial number and token scheme type.
     * @param aliasId The alias that controls the foundry.
     * @param serialNumber The serial number of the foundry.
     * @param tokenSchemeType The tokenSchemeType of the foundry.
     * @returns The tokenId.
     */
    static computeTokenId(aliasId, serialNumber, tokenSchemeType) {
        return (0, bindings_1.callUtilsMethod)({
            name: 'computeTokenId',
            data: {
                aliasId,
                serialNumber,
                tokenSchemeType,
            },
        });
    }
    /**
     * Returns a valid Address parsed from a String.
     */
    static parseBech32Address(address) {
        const addr = (0, bindings_1.callUtilsMethod)({
            name: 'parseBech32Address',
            data: {
                address,
            },
        });
        return types_1.Address.parse(addr);
    }
    /**
     * Returns a block ID (Blake2b256 hash of the block bytes)
     */
    static blockId(block) {
        return (0, bindings_1.callUtilsMethod)({
            name: 'blockId',
            data: {
                block,
            },
        });
    }
    /**
     * Returns a Milestone ID (Blake2b256 hash of the milestone essence)
     */
    static milestoneId(payload) {
        return (0, bindings_1.callUtilsMethod)({
            name: 'milestoneId',
            data: {
                payload,
            },
        });
    }
    /**
     * Returns the transaction ID (Blake2b256 hash of the provided transaction payload)
     * @param payload The transaction payload.
     * @returns The transaction id.
     */
    static transactionId(payload) {
        return (0, bindings_1.callUtilsMethod)({
            name: 'transactionId',
            data: {
                payload,
            },
        });
    }
    /**
     * Transforms bech32 to hex.
     */
    static bech32ToHex(bech32) {
        return (0, bindings_1.callUtilsMethod)({
            name: 'bech32ToHex',
            data: {
                bech32,
            },
        });
    }
    /**
     * Transforms a hex encoded address to a bech32 encoded address.
     */
    static hexToBech32(hex, bech32Hrp) {
        return (0, bindings_1.callUtilsMethod)({
            name: 'hexToBech32',
            data: {
                hex,
                bech32Hrp,
            },
        });
    }
    /**
     * Transforms an alias id to a bech32 encoded address.
     */
    static aliasIdToBech32(aliasId, bech32Hrp) {
        return (0, bindings_1.callUtilsMethod)({
            name: 'aliasIdToBech32',
            data: {
                aliasId,
                bech32Hrp,
            },
        });
    }
    /**
     * Transforms an nft id to a bech32 encoded address.
     */
    static nftIdToBech32(nftId, bech32Hrp) {
        return (0, bindings_1.callUtilsMethod)({
            name: 'nftIdToBech32',
            data: {
                nftId,
                bech32Hrp,
            },
        });
    }
    /**
     * Transforms a hex encoded public key to a bech32 encoded address.
     */
    static hexPublicKeyToBech32Address(hex, bech32Hrp) {
        return (0, bindings_1.callUtilsMethod)({
            name: 'hexPublicKeyToBech32Address',
            data: {
                hex,
                bech32Hrp,
            },
        });
    }
    /**
     * Checks if a String is a valid bech32 encoded address.
     */
    static isAddressValid(address) {
        return (0, bindings_1.callUtilsMethod)({
            name: 'isAddressValid',
            data: {
                address,
            },
        });
    }
    /**
     * Compute the hash of a transaction essence.
     */
    static hashTransactionEssence(essence) {
        return (0, bindings_1.callUtilsMethod)({
            name: 'hashTransactionEssence',
            data: {
                essence,
            },
        });
    }
    /**
     * Verifies an ed25519 signature against a message.
     */
    static verifyEd25519Signature(signature, message) {
        return (0, bindings_1.callUtilsMethod)({
            name: 'verifyEd25519Signature',
            data: {
                signature,
                message,
            },
        });
    }
    /**
     * Verifies a Secp256k1Ecdsa signature against a message.
     */
    static verifySecp256k1EcdsaSignature(publicKey, signature, message) {
        return (0, bindings_1.callUtilsMethod)({
            name: 'verifySecp256k1EcdsaSignature',
            data: {
                publicKey,
                signature,
                message,
            },
        });
    }
    /**
     * Verify if a mnemonic is a valid BIP39 mnemonic.
     */
    static verifyMnemonic(mnemonic) {
        return (0, bindings_1.callUtilsMethod)({
            name: 'verifyMnemonic',
            data: { mnemonic },
        });
    }
}
exports.Utils = Utils;
//# sourceMappingURL=utils.js.map