import { ClientMethodHandler } from './client-method-handler';
import { IClientOptions, IGenerateAddressesOptions, IBuildBlockOptions, QueryParameter, PreparedTransactionData, INetworkInfo, INode, IAuth, BasicOutputBuilderParams, AliasOutputBuilderParams, FoundryOutputBuilderParams, NftOutputBuilderParams, FoundryQueryParameter, NftQueryParameter, AliasQueryParameter } from '../types/client';
import type { INodeInfoWrapper } from '../types/client/nodeInfo';
import { Bip44, SecretManagerType } from '../types/secret_manager/secret-manager';
import { AliasOutput, BasicOutput, FoundryOutput, NftOutput, Block, BlockId, UnlockCondition, Payload, TransactionPayload, MilestonePayload, TreasuryOutput, Output } from '../types/block';
import { HexEncodedString } from '../utils';
import { IBlockMetadata, INodeInfo, INodeInfoProtocol, IPeer, UTXOInput, OutputId } from '../types';
import { IMilestoneUtxoChangesResponse, OutputResponse, ReceiptsResponse, IOutputsResponse } from '../types/models/api';
/** The Client to interact with nodes. */
export declare class Client {
    private methodHandler;
    constructor(options: IClientOptions | ClientMethodHandler);
    destroy(): Promise<any>;
    /**
     * Returns the node information together with the url of the used node
     * @returns { Promise<INodeInfoWrapper> }.
     */
    getInfo(): Promise<INodeInfoWrapper>;
    /**
     * Gets the network related information such as network_id and min_pow_score
     */
    getNetworkInfo(): Promise<INetworkInfo>;
    /** Fetch basic output IDs based on query parameters */
    basicOutputIds(queryParameters: QueryParameter[]): Promise<IOutputsResponse>;
    /** Get output from a known outputID */
    getOutput(outputId: OutputId): Promise<OutputResponse>;
    /** Fetch OutputResponse from provided OutputIds (requests are sent in parallel) */
    getOutputs(outputIds: OutputId[]): Promise<OutputResponse[]>;
    /** Build and post a block */
    buildAndPostBlock(secretManager?: SecretManagerType, options?: IBuildBlockOptions): Promise<[BlockId, Block]>;
    /**
     * Returns tips that are ideal for attaching a block.
     * The tips can be considered as non-lazy and are therefore ideal for attaching a block.
     */
    getTips(): Promise<BlockId[]>;
    /**
     * Post block in JSON format, returns the block ID.
     */
    postBlock(block: Block): Promise<BlockId>;
    /**
     * Get block as JSON.
     */
    getBlock(blockId: BlockId): Promise<Block>;
    /**
     * Get block metadata.
     */
    getBlockMetadata(blockId: BlockId): Promise<IBlockMetadata>;
    /**
     * Find inputs from addresses for a provided amount (useful for offline signing)
     */
    findInputs(addresses: string[], amount: bigint): Promise<UTXOInput[]>;
    /**
     * Prepare a transaction for signing
     */
    prepareTransaction(secretManager?: SecretManagerType, options?: IBuildBlockOptions): Promise<PreparedTransactionData>;
    /**
     * Sign a transaction
     */
    signTransaction(secretManager: SecretManagerType, preparedTransactionData: PreparedTransactionData): Promise<TransactionPayload>;
    /**
     * Create a signature unlock using the provided `secretManager`.
     */
    signatureUnlock(secretManager: SecretManagerType, transactionEssenceHash: HexEncodedString, chain: Bip44): Promise<UnlockCondition>;
    /**
     * Submit a payload in a block
     */
    postBlockPayload(payload: Payload): Promise<[BlockId, Block]>;
    /**
     * Get a node candidate from the healthy node pool.
     */
    getNode(): Promise<INode>;
    /**
     * Get the network id of the node we're connecting to.
     */
    getNetworkId(): Promise<number>;
    /**
     * Returns the bech32_hrp.
     */
    getBech32Hrp(): Promise<string>;
    /**
     * Returns the min PoW score.
     */
    getMinPowScore(): Promise<number>;
    /**
     * Returns the tips interval.
     */
    getTipsInterval(): Promise<number>;
    /**
     * Returns the token supply.
     */
    getTokenSupply(): Promise<string>;
    /**
     * Returns the protocol parameters.
     */
    getProtocolParameters(): Promise<INodeInfoProtocol>;
    /**
     * Returns if local pow should be used or not.
     */
    getLocalPow(): Promise<boolean>;
    /**
     * Get fallback to local proof of work timeout.
     */
    getFallbackToLocalPow(): Promise<boolean>;
    /**
     * Get health of node by input url.
     */
    getHealth(url: string): Promise<boolean>;
    /**
     * Get info of node with input url.
     */
    getNodeInfo(url: string, auth?: IAuth): Promise<INodeInfo>;
    /**
     * Get peers.
     */
    getPeers(): Promise<IPeer[]>;
    /**
     * Post block as raw bytes, returns the block ID.
     */
    postBlockRaw(block: Block): Promise<BlockId>;
    /**
     * Get block as raw bytes.
     */
    getBlockRaw(blockId: BlockId): Promise<Uint8Array>;
    /**
     * Look up a milestone by a given milestone index.
     */
    getMilestoneById(milestoneId: string): Promise<MilestonePayload>;
    /**
     * Returns all UTXO changes that happened at a specific milestone.
     */
    getUtxoChangesById(milestoneId: string): Promise<IMilestoneUtxoChangesResponse>;
    /**
     * Look up a milestone by a given milestone index.
     */
    getMilestoneByIndex(index: number): Promise<MilestonePayload>;
    /**
     * Returns all UTXO changes that happened at a specific milestone.
     */
    getUtxoChangesByIndex(index: number): Promise<IMilestoneUtxoChangesResponse>;
    /**
     * Get receipts.
     */
    getReceipts(): Promise<ReceiptsResponse>;
    /**
     * Get the receipts by the given milestone index.
     */
    getReceiptsMigratedAt(milestoneIndex: number): Promise<ReceiptsResponse[]>;
    /**
     * Get the treasury output.
     */
    getTreasury(): Promise<TreasuryOutput>;
    /**
     * Returns the included block of the transaction.
     */
    getIncludedBlock(transactionId: string): Promise<Block>;
    /**
     * Returns the metadata of the included block of the transaction.
     */
    getIncludedBlockMetadata(transactionId: string): Promise<Block>;
    /**
     * Transforms a hex encoded address to a bech32 encoded address.
     */
    hexToBech32(hex: string, bech32Hrp?: string): Promise<string>;
    /**
     * Transforms an alias id to a bech32 encoded address.
     */
    aliasIdToBech32(aliasId: string, bech32Hrp?: string): Promise<string>;
    /**
     * Transforms an nft id to a bech32 encoded address.
     */
    nftIdToBech32(nftId: string, bech32Hrp?: string): Promise<string>;
    /**
     * Transforms a hex encoded public key to a bech32 encoded address.
     */
    hexPublicKeyToBech32Address(hex: string, bech32Hrp?: string): Promise<string>;
    /**
     * Fetch alias output IDs
     */
    aliasOutputIds(queryParameters: AliasQueryParameter[]): Promise<IOutputsResponse>;
    /**
     * Fetch alias output ID
     */
    aliasOutputId(aliasId: string): Promise<string>;
    /**
     * Fetch NFT output IDs
     */
    nftOutputIds(queryParameters: NftQueryParameter[]): Promise<IOutputsResponse>;
    /**
     * Fetch NFT output ID
     */
    nftOutputId(nftId: string): Promise<string>;
    /**
     * Fetch Foundry Output IDs
     */
    foundryOutputIds(queryParameters: FoundryQueryParameter[]): Promise<IOutputsResponse>;
    /**
     * Fetch Foundry Output ID
     */
    foundryOutputId(foundryId: string): Promise<string>;
    /**
     * Try to get OutputResponse from provided OutputIds (requests are sent
     * in parallel and errors are ignored, can be useful for spent outputs)
     */
    getOutputsIgnoreErrors(outputIds: string[]): Promise<OutputResponse[]>;
    /**
     * Find all blocks by provided block IDs.
     */
    findBlocks(blockIds: BlockId[]): Promise<Block[]>;
    /**
     * Retries (promotes or reattaches) a block for provided block id. Block should be
     * retried only if they are valid and haven't been confirmed for a while.
     */
    retry(blockId: BlockId): Promise<[BlockId, Block]>;
    /**
     * Retries (promotes or reattaches) a block for provided block id until it's included (referenced by a
     * milestone). Default interval is 5 seconds and max attempts is 40. Returns the included block at first
     * position and additional reattached blocks
     */
    retryUntilIncluded(blockId: BlockId, interval?: number, maxAttempts?: number): Promise<[BlockId, Block][]>;
    /**
     * Function to consolidate all funds from a range of addresses to the address with the lowest index in that range
     * Returns the address to which the funds got consolidated, if any were available
     */
    consolidateFunds(secretManager: SecretManagerType, generateAddressesOptions: IGenerateAddressesOptions): Promise<string>;
    /**
     * Reattaches blocks for provided block id. Blocks can be reattached only if they are valid and haven't been
     * confirmed for a while.
     */
    reattach(blockId: BlockId): Promise<[BlockId, Block]>;
    /**
     * Reattach a block without checking if it should be reattached
     */
    reattachUnchecked(blockId: BlockId): Promise<[BlockId, Block]>;
    /**
     * Promotes a block. The method should validate if a promotion is necessary through get_block. If not, the
     * method should error out and should not allow unnecessary promotions.
     */
    promote(blockId: BlockId): Promise<[BlockId, Block]>;
    /**
     * Promote a block without checking if it should be promoted
     */
    promoteUnchecked(blockId: BlockId): Promise<[BlockId, Block]>;
    /**
     * Returns the unhealthy nodes.
     */
    unhealthyNodes(): Promise<Set<INode>>;
    /**
     * Build a Basic Output.
     */
    buildBasicOutput(params: BasicOutputBuilderParams): Promise<BasicOutput>;
    /**
     * Build an Alias Output.
     */
    buildAliasOutput(params: AliasOutputBuilderParams): Promise<AliasOutput>;
    /**
     * Build a Foundry Output.
     */
    buildFoundryOutput(params: FoundryOutputBuilderParams): Promise<FoundryOutput>;
    /**
     * Build an Nft Output.
     */
    buildNftOutput(params: NftOutputBuilderParams): Promise<NftOutput>;
    /**
     * Listen to MQTT topics.
     */
    listenMqtt(topics: string[], callback: (error: Error, result: string) => void): Promise<void>;
    /**
     * Stop listening for provided MQTT topics.
     */
    clearMqttListeners(topics: string[]): Promise<void>;
    /**
     * Calculate the minimum required storage deposit for an output.
     * @param output output to calculate the deposit amount for.
     * @returns The amount.
     */
    minimumRequiredStorageDeposit(output: Output): Promise<number>;
    /**
     * Request funds from a faucet, for example `https://faucet.testnet.shimmer.network/api/enqueue` or `http://localhost:8091/api/enqueue`.
     */
    requestFundsFromFaucet(url: string, address: string): Promise<string>;
    /**
     * Extension method which provides request methods for plugins.
     * @param basePluginPath The base path for the plugin eg indexer/v1/ .
     * @param method The http method.
     * @param endpoint The path for the plugin request.
     * @param queryParams Additional query params for the request.
     * @param request The request object.
     * @returns The response json.
     */
    callPluginRoute(basePluginPath: string, method: 'GET' | 'POST', endpoint: string, queryParams?: string[], request?: string): Promise<string>;
}
